@@ -5,11 +5,10 @@ import os
 import re
 import socket
 
-from stackifyapm.conf.constants import DEFAULT_CONFIG_FILE
-from stackifyapm.conf.constants import LOG_PATH
+from stackifyapm.conf import constants
 from stackifyapm.utils import compat
 
-__all__ = ("setup_logging", "Config")
+__all__ = ("setup_logging", "Config", "setup_stackifyapm_logging")
 
 
 class ConfigError(ValueError):
@@ -26,7 +25,7 @@ class _ConfigValue(object):
         self.default = default
         self.required = required
 
-        self.env_key = env_key or "STACKIFY_APM_" + dict_key
+        self.env_key = env_key or "STACKIFY_" + dict_key
 
     def __get__(self, instance, owner):
         return instance._values.get(self.dict_key, self.default) if instance else self.default
@@ -94,10 +93,12 @@ class _ConfigBase(object):
             new_value = self._NO_VALUE
             if config_value.env_key and config_value.env_key in env_dict:
                 new_value = env_dict[config_value.env_key]
-            elif field in inline_dict:
-                new_value = inline_dict[field]
             elif config_value.dict_key in config_dict:
                 new_value = config_dict[config_value.dict_key]
+            elif field in inline_dict or field.upper() in inline_dict:
+                new_value = inline_dict.get(field) or inline_dict.get(field.upper())
+                if new_value is None:
+                    new_value = config_value.default
 
             if new_value is not self._NO_VALUE:
                 try:
@@ -112,7 +113,6 @@ class _ConfigBase(object):
 
 class Config(_ConfigBase):
     service_name = _ConfigValue("SERVICE_NAME", validators=[RegexValidator("^[a-zA-Z0-9 _-]+$")], required=True)
-    environment = _ConfigValue("ENVIRONMENT", default='Production')
     hostname = _ConfigValue("HOSTNAME", default=socket.gethostname())
     capture_body = _ConfigValue("CAPTURE_BODY", default="off")
     async_mode = _BoolConfigValue("ASYNC_MODE", default=True)
@@ -121,11 +121,26 @@ class Config(_ConfigBase):
     framework_name = _ConfigValue("FRAMEWORK_NAME", default=None)
     framework_version = _ConfigValue("FRAMEWORK_VERSION", default=None)
     instrument = _BoolConfigValue("DISABLE_INSTRUMENTATION", default=True)
-    application_name = _ConfigValue("APPLICATION_NAME", default='Python Application')
     base_dir = _ConfigValue("BASE_DIR", default=None)
-    config_file = _ConfigValue("CONFIG_FILE", default=DEFAULT_CONFIG_FILE)
-    multiprocessing = _BoolConfigValue("MULTIPROCESSING", default=False)
-    rum_enabled = _BoolConfigValue("RUM_ENABLED", default=False)
+    application_name = _ConfigValue("APPLICATION_NAME", default=constants.DEFAULT_APPLICATION_NAME)
+    environment = _ConfigValue("ENVIRONMENT", env_key="STACKIFY_ENVIRONMENT_NAME", default=constants.DEFAULT_ENVIRONMENT)
+    config_file = _ConfigValue("CONFIG_FILE", default=constants.DEFAULT_CONFIG_FILE)
+    multiprocessing = _BoolConfigValue("MULTIPROCESSING", default=constants.DEFAULT_MULTIPROCESSING)
+    rum_enabled = _BoolConfigValue("RUM_ENABLED", env_key="STACKIFY_RUM", default=constants.DEFAULT_RUM_ENABLED)
+    rum_auto_injection = _BoolConfigValue("RUM_AUTO_INJECTION", env_key="STACKIFY_RUM_AUTO_INJECT", default=constants.DEFAULT_RUM_AUTO_INJECTION)
+    socket_url = _ConfigValue("SOCKET_URL", env_key="STACKIFY_TRANSPORT_SOCKET_PATH", default=constants.DEFAULT_TRANSPORT_SOCKET_FILE)
+    transport = _ConfigValue("TRANSPORT", default=constants.DEFAULT_TRANSPORT)
+    http_endpoint = _ConfigValue(
+        "HTTP_ENDPOINT",
+        env_key="STACKIFY_TRANSPORT_HTTP_ENDPOINT",
+        validators=[RegexValidator("http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+")],
+        default=constants.DEFAULT_HTTP_ENDPOINT,
+    )
+    log_path = _ConfigValue("LOG_PATH", env_key="STACKIFY_TRANSPORT_LOG_PATH", default=constants.LOG_PATH)
+    debug = _BoolConfigValue("DEBUG", default=constants.DEFAULT_DEBUG)
+    queue = _BoolConfigValue("QUEUE", default=constants.DEFAULT_QUEUE)
+    lambda_handler = _ConfigValue("LAMBDA_HANDLER", default="")
+    prefix_enabled = _BoolConfigValue("PREFIX_ENABLED", default=constants.DEFAULT_PREFIX_ENABLED)
 
 
 class IncrementalFileHandler(logging.handlers.RotatingFileHandler):
@@ -195,17 +210,39 @@ class StackifyFormatter(logging.Formatter):
 
 
 def setup_logging(client):
+    """
+    Configure debug loggings for Stackify APM
+    """
+    try:
+        for logger_name in ["stackifyapm", "stackifyapm.traces", "stackifyapm.errors", "stackifyapm.instrument"]:
+            _logger = logging.getLogger(logger_name)
+            _logger.propagate = False
+            _logger.setLevel(logging.DEBUG if client.config.debug else logging.INFO)
+            _logger_name = "{}.{}".format(logger_name, "error" in logger_name and "err" or "out")
+
+            if os.path.exists(constants.LOG_PATH):
+                _handler = logging.FileHandler("{}{}".format(constants.LOG_PATH, _logger_name))
+            else:
+                _handler = logging.StreamHandler()
+
+            _logger.addHandler(_handler)
+
+    except Exception:
+        pass
+
+
+def setup_stackifyapm_logging(client):
     """
     Configure loggings to pipe to Stackify Linux Agent
     """
     logger = None
     host_name = client.get_system_info().get("hostname")
     process_id = client.get_process_info().get("pid")
-    filename = "{}{}#{}-1.log".format(LOG_PATH, host_name, process_id)
+    filename = "{}{}#{}-1.log".format(client.config.log_path, host_name, process_id)
 
     def namer(name):
         num = int(name.split("/")[-1].split(".")[-1] or 0) + 1
-        return "{}{}#{}-{}.log".format(LOG_PATH, host_name, process_id, num)
+        return "{}{}#{}-{}.log".format(client.config.log_path, host_name, process_id, num)
 
     try:
         handler = IncrementalFileHandler(filename, maxBytes=50000000)
@@ -221,16 +258,7 @@ def setup_logging(client):
         # Set stackify-python-apm first log file to chmod 777
         os.chmod(filename, 0o777)
 
-        for logger_name in ["stackifyapm", "stackifyapm.traces", "stackifyapm.errors", "stackifyapm.instrument"]:
-            _logger = logging.getLogger(logger_name)
-            _logger.propagate = False
-            _logger.setLevel(logging.DEBUG)
-
-            if "error" in logger_name:
-                _logger.addHandler(logging.FileHandler("{}{}.err".format(LOG_PATH, logger_name)))
-            else:
-                _logger.addHandler(logging.FileHandler("{}{}.out".format(LOG_PATH, logger_name)))
     except Exception:
-        print("No such file or directory: '/usr/local/stackify/stackify-python-apm/log/'")
+        print("No such file or directory: {}.".format(client.config.log_path))
 
     return logger
