@@ -112,13 +112,13 @@ class Transaction(object):
     def get_exceptions(self):
         return self.exceptions
 
-    def begin_span(self, name, span_type, context=None, leaf=False, is_async=False):
+    def begin_span(self, span_type, context=None, leaf=False, is_async=False):
         parent_span = execution_context.get_span()
 
         if parent_span and parent_span.leaf:
             span = DroppedSpan(parent_span, leaf=True)
         else:
-            span = Span(transaction=self, name=name, span_type=span_type, context=context, leaf=leaf, is_async=is_async)
+            span = Span(transaction=self, span_type=span_type, context=context, leaf=leaf, is_async=is_async)
             span.parent = parent_span
 
         if not isinstance(span, DroppedSpan):
@@ -128,12 +128,15 @@ class Transaction(object):
         return span
 
     def update_span_context(self, span_id, span_context):
-        spans = [span for span in self.spans if span.id == span_id]
-        if spans:
-            span = spans[0]
-            span.context.update(span_context)
-        else:
-            logger.warning("Can't find span id: {} from transaction id: {}".format(span_id, self.id))
+        try:
+            spans = [span for span in self.spans if span.id == span_id]
+            if spans:
+                span = spans[0]
+                span.context.update(span_context)
+            else:
+                logger.warning("Can't find span id: {} from transaction id: {}".format(span_id, self.id))
+        except Exception as e:
+            error_logger.error("Update span context error: {}.".format(e))
 
     def end_span(self):
         span = execution_context.get_span()
@@ -152,7 +155,7 @@ class Transaction(object):
     def is_done(self):
         return all([span.end_time for span in self.spans])
 
-    def to_dict(self):
+    def get_stacks(self):
         stacks = []
         spans = [span.to_dict() for span in self.spans]
         spans.reverse()
@@ -167,13 +170,16 @@ class Transaction(object):
             else:
                 stacks.insert(0, spans_kv.pop(span['id']))
 
+        return stacks
+
+    def to_dict(self):
         result = {
             "id": self.id,
             "call": self._get_call(),
             "reqBegin": self.start_time,
             "reqEnd": self.end_time,
             "props": self._get_props(),
-            "stacks": stacks,
+            "stacks": self.get_stacks(),
         }
         if self.trace_parent.span_id and self.trace_parent.span_id != self.id:
             result["parent_id"] = self.trace_parent.span_id
@@ -181,7 +187,7 @@ class Transaction(object):
         if self.exceptions:
             result["exceptions"] = self.exceptions
 
-        return json.dumps(result)
+        return result
 
     def _get_call(self):
         call = encoding.keyword_field(self.name or "").split(" ")[-1]
@@ -201,22 +207,23 @@ class Transaction(object):
         application_info = self._meta_data.get('application_info')
 
         base_object = {
+            "PROFILER_VERSION": VERSION,
             "CATEGORY": service_info["language"]["name"].title(),
             "THREAD_ID": self._thread_id,
             "TRACE_ID": self.trace_parent.trace_id,
             "TRACE_SOURCE": "PYTHON",
             "TRACE_TARGET": "RETRACE",
-            "TRACE_VERSION": "{}".format(VERSION or TRACE_CONTEXT_VERSION or "unknown"),
+            "TRACE_VERSION": "{}".format(TRACE_CONTEXT_VERSION or "unknown"),
             "HOST_NAME": system_info["hostname"],
             "OS_TYPE": system_info["platform"].upper(),
-            "PROCESS_ID": process_info["pid"],
+            "PROCESS_ID": str(process_info["pid"]),
             "APPLICATION_PATH": "/",
             "APPLICATION_FILESYSTEM_PATH": application_info["base_dir"],
             "APPLICATION_NAME": application_info["application_name"],
             "APPLICATION_ENV": application_info["environment"],
         }
 
-        if self.transaction_type in ['custom']:
+        if self.transaction_type in ['custom', 'lambda', 'azure']:
             base_object["TRACETYPE"] = "TASK"
         else:
             base_object["TRACETYPE"] = "WEBAPP"
@@ -224,12 +231,41 @@ class Transaction(object):
         if "request" in self.context:
             base_object["METHOD"] = self.context["request"]["method"]
             base_object["URL"] = self.context["request"]["url"]["full"]
+
+            if "body" in self.context["request"]:
+                base_object["REQUEST_BODY"] = str(self.context["request"]["body"])
+                base_object["REQUEST_SIZE_BYTES"] = str(self.context["request"]["body_size"])
+
+            if "headers" in self.context["request"]:
+                base_object["REQUEST_HEADERS"] = json.dumps([
+                    {key: val} for key, val
+                    in iteritems(self.context["request"]["headers"])
+                ])
+
+        if "reporting_url" in self.context:
+            base_object["REPORTING_URL"] = self.context["reporting_url"]
+        elif "request" in self.context:
             base_object["REPORTING_URL"] = self.context["request"]["url"]["pathname"]
         else:
             base_object["REPORTING_URL"] = self._get_call()
 
         if "response" in self.context:
-            base_object["STATUS"] = self.context["response"]["status_code"]
+            base_object["STATUS"] = str(self.context["response"]["status_code"])
+
+            if "body" in self.context["response"]:
+                base_object["RESPONSE_BODY"] = str(self.context["response"]["body"])
+                base_object["RESPONSE_SIZE_BYTES"] = str(self.context["response"]["body_size"])
+            if "headers" in self.context["response"]:
+                base_object["RESPONSE_HEADERS"] = json.dumps([
+                    {key: val} for key, val
+                    in iteritems(self.context["response"]["headers"])
+                ])
+
+        if "rum" in self.context:
+            base_object["RUM"] = self.context["rum"] and 'TRUE' or 'FALSE'
+
+        if "lambda" in self.context:
+            base_object["AWS_LAMBDA_ARN"] = self.context["lambda"]["invoked_function_arn"]
 
         return base_object
 
@@ -238,7 +274,6 @@ class Span(object):
     __slots__ = (
         "id",
         "transaction",
-        "name",
         "type",
         "context",
         "leaf",
@@ -249,13 +284,12 @@ class Span(object):
         "_thread_id",
     )
 
-    def __init__(self, transaction, name, span_type, context=None, leaf=False, is_async=False):
+    def __init__(self, transaction, span_type, context=None, leaf=False, is_async=False):
         """
         Create a new Span
         """
         self.id = "{}".format(len(transaction.get_spans()) + 1)
         self.transaction = transaction
-        self.name = name
         self.type = span_type
         self.context = context
         self.leaf = leaf
@@ -292,7 +326,7 @@ class Span(object):
         if not self.context:
             return base_object
 
-        if self.context.get("type", "").lower() not in ["cassandra", "stackify"] and self.type.split('.')[0].lower() not in ["custom"]:
+        if self.context.get("type", "").lower() not in ["cassandra", "stackify", "log"] and self.type.split('.')[0].lower() not in ["custom"]:
             base_object["COMPONENT_CATEGORY"] = self._get_component_category()
             base_object["COMPONENT_DETAIL"] = self._get_component_detail()
 
@@ -301,7 +335,11 @@ class Span(object):
 
         if self.context.get("type", "").lower() in ["database", "cassandra"]:
             if "statement" in self.context:
-                base_object["SQL"] = self.context["statement"]
+                if len(self.context["statement"]) > constants.SQL_STATEMENT_MAX_LENGTH:
+                    base_object["SQL"] = self.context["statement"][:constants.SQL_STATEMENT_MAX_LENGTH]
+                    base_object["SQL_TRUNCATED"] = True
+                else:
+                    base_object["SQL"] = self.context["statement"]
 
         if self.context.get("type", "").lower() == "mongodb":
             if "collection" in self.context:
@@ -311,7 +349,7 @@ class Span(object):
             base_object["URL"] = self.context["url"]
 
         if "status_code" in self.context:
-            base_object["STATUS"] = self.context["status_code"]
+            base_object["STATUS"] = str(self.context["status_code"])
 
         if "request_method" in self.context:
             base_object["METHOD"] = self.context["request_method"].upper()
@@ -341,6 +379,30 @@ class Span(object):
         if self.context.get("log_id"):
             base_object["ID"] = self.context["log_id"]
 
+        if "request_body" in self.context:
+            base_object["REQUEST_BODY"] = str(self.context["request_body"])
+            base_object["REQUEST_SIZE_BYTES"] = str(self.context["request_body_size"])
+
+        if "request_headers" in self.context:
+            base_object["REQUEST_HEADERS"] = str(self.context["request_headers"])
+
+        if "response_body" in self.context:
+            base_object["RESPONSE_BODY"] = str(self.context["response_body"])
+            base_object["RESPONSE_SIZE_BYTES"] = str(self.context["response_body_size"])
+
+        if "response_headers" in self.context:
+            base_object["RESPONSE_HEADERS"] = str(self.context["response_headers"])
+
+        # Logger
+        if "level" in self.context:
+            base_object["LEVEL"] = str(self.context["level"])
+
+        if "message" in self.context:
+            base_object["MESSAGE"] = str(self.context["message"])
+
+        if "exception" in self.context:
+            base_object["EXCEPTION"] = str(self.context["exception"])
+
         return base_object
 
     def _get_category(self):
@@ -387,14 +449,19 @@ class Tracer(object):
         try:
             self.manager.start()
         except Exception as e:
-            logger.debug("Error when starting manager. Details: {}".format(e))
+            error_logger.debug("Error when starting manager. Details: {}".format(e))
 
     def begin_transaction(self, transaction_type, trace_parent=None, client=None):
         """
         Start a new transactions and bind it in a thread-local variable
         """
         meta_data = client and client.get_meta_data() or {}
-        transaction = self.manager.Transaction(transaction_type, trace_parent, meta_data=meta_data)
+
+        if client and client.config.multiprocessing:
+            transaction = self.manager.Transaction(transaction_type, trace_parent, meta_data=meta_data)
+        else:
+            transaction = Transaction(transaction_type, trace_parent, meta_data=meta_data)
+
         if trace_parent is None:
             transaction.set_trace_parent(TraceParent(
                 constants.TRACE_CONTEXT_VERSION,
@@ -418,7 +485,10 @@ class Tracer(object):
                 else:
                     self.queue(transaction)
             else:
-                logger.debug("Dropped transaction {}".format(transaction.get_id()))
+                try:
+                    error_logger.debug("Dropped transaction: {}.".format(transaction.to_dict()))
+                except Exception:
+                    error_logger.debug("Dropped transaction: {}.".format(transaction.get_id()))
 
         return transaction
 
@@ -432,18 +502,16 @@ class Tracer(object):
 
 
 class CaptureSpan(object):
-    def __init__(self, name=None, span_type="custom", extra={}, leaf=False, is_async=False, **kwargs):
+    def __init__(self, span_type="custom", extra={}, leaf=False, is_async=False, **kwargs):
         """
         Start a new CaptureSpan
         """
-        self.name = name
         self.type = span_type
         self.extra = extra
         self.leaf = leaf
         self.is_async = is_async
 
     def __call__(self, func):
-        self.name = self.name
 
         @wraps(func)
         def wrapper(*args, **kwds):
@@ -462,17 +530,23 @@ class CaptureSpan(object):
         return wrapper
 
     def __enter__(self):
-        transaction = execution_context.get_transaction()
-        if transaction and transaction.get_is_sampled():
-            return transaction.begin_span(self.name, self.type, context=self.extra, leaf=self.leaf, is_async=self.is_async)
+        try:
+            transaction = execution_context.get_transaction()
+            if transaction and transaction.get_is_sampled():
+                return transaction.begin_span(self.type, context=self.extra, leaf=self.leaf, is_async=self.is_async)
+        except Exception as e:
+            error_logger.error('Start span error: {}'.format(e))
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        transaction = execution_context.get_transaction()
-        if transaction and transaction.get_is_sampled():
-            try:
-                transaction.end_span()
-            except LookupError:
-                error_logger.info("ended non-existing span {} of type {}".format(self.name, self.type))
+        try:
+            transaction = execution_context.get_transaction()
+            if transaction and transaction.get_is_sampled():
+                try:
+                    transaction.end_span()
+                except LookupError:
+                    error_logger.info("ended non-existing span {} of type {}".format(self.name, self.type))
+        except Exception as e:
+            error_logger.error('End span error: {}'.format(e))
 
 
 def set_transaction_name(name, override=True):
